
package de.hsrm.ads;

import static org.junit.Assert.*;

import java.util.List;

import org.junit.Test;

public class SearchTreeTest {

	SearchTree<Integer,String> simpsons;
	SearchTree<Integer,String> empty;
	SearchTree<Integer,String> left;
	SearchTree<Integer,String> right;
	SearchTree<Integer,String> rootOnly;
	SearchTree<Integer,String> duplicates;
	
	public void setup() {
	
		// Simpsons tree
		simpsons = new SearchTree<Integer,String>();
		String[] values = { "lisa", "homer", "maggie", "marge", "bart", "moe" };
		int[] keys = { 8, 42, 1, 39, 10, 42 };
		for (int i = 0; i < values.length; ++i) {
			simpsons.insert(keys[i], values[i]);
		}

		// empty tree
		empty = new SearchTree<Integer,String>();

		// left
		left = new SearchTree<Integer,String>();
		left.insert(4, "4");
		left.insert(3, "3");
		left.insert(2, "2");
		left.insert(1, "1");

		// right
		right = new SearchTree<Integer,String>();
		right.insert(1, "1");
		right.insert(2, "2");
		right.insert(3, "3");
		right.insert(4, "4");

		// root only
		rootOnly = new SearchTree<Integer,String>();
		rootOnly.insert(1, "1");

		// duplicates
		duplicates = new SearchTree<Integer,String>();
		duplicates.insert(2, "2");
		duplicates.insert(1, "1");
		duplicates.insert(3, "3");
		duplicates.insert(2, "2");
		duplicates.insert(2, "2");

	}
		
	
	@Test
	public void testSimpsons() {
		setup();
		assertEquals ( "Simpsons tree not correct.", (int)simpsons.root.key, 8 );
		assertEquals ( "Simpsons tree not correct.", (int)simpsons.root.left.key, 1 );
		assertEquals ( "Simpsons tree not correct.", simpsons.root.left.left, null );
		assertEquals ( "Simpsons tree not correct.", simpsons.root.left.right, null );
		assertEquals ( "Simpsons tree not correct.", (int)simpsons.root.right.key, 42 );
		assertEquals ( "Simpsons tree not correct.", (int)simpsons.root.right.left.key, 39 );
		assertEquals ( "Simpsons tree not correct.", (int)simpsons.root.right.left.left.key, 10 );
	}

	@Test
	public void testEmpty() {
		setup();
		assertEquals ( "Empty tree should have 'null' root.", empty.root, null );
		assertEquals(0, empty.countSmaller(-1000));
	}

	@Test
	public void testRootOnly() {
		setup();
		assertEquals ( "Root-only tree incorrect.", (int)rootOnly.root.key, 1 );
		assertEquals(0, rootOnly.countSmaller(0));
		assertEquals(0, rootOnly.countSmaller(1));
		assertEquals(1, rootOnly.countSmaller(2));
	}

	@Test
	public void testLeft() {
		setup();
		assertEquals ( "Left-hanging tree 1<-2<-3<-4 incorrect.", (int)left.root.key, 4 );
		assertEquals ( "Left-hanging tree 1<-2<-3<-4 incorrect.", left.root.right, null );
		assertEquals ( "Left-hanging tree 1<-2<-3<-4 incorrect.", (int)left.root.left.key, 3 );
		assertEquals ( "Left-hanging tree 1<-2<-3<-4 incorrect.", left.root.left.right, null );
		assertEquals ( "Left-hanging tree 1<-2<-3<-4 incorrect.", (int)left.root.left.left.key, 2 );
		assertEquals ( "Left-hanging tree 1<-2<-3<-4 incorrect.", left.root.left.left.right, null );
		assertEquals ( "Left-hanging tree 1<-2<-3<-4 incorrect.", (int)left.root.left.left.left.key, 1 );
		assertEquals ( "Left-hanging tree 1<-2<-3<-4 incorrect.", left.root.left.left.left.left, null );
		assertEquals ( "Left-hanging tree 1<-2<-3<-4 incorrect.", left.root.left.left.left.right, null );
	}

	@Test
	public void testRight() {
		setup();
		assertEquals ( "Right-hanging tree 1->2->3->4 incorrect.", (int)right.root.key, 1 );
		assertEquals ( "Right-hanging tree 1->2->3->4 incorrect.", right.root.left, null );
		assertEquals ( "Right-hanging tree 1->2->3->4 incorrect.", (int)right.root.right.key, 2 );
		assertEquals ( "Right-hanging tree 1->2->3->4 incorrect.", right.root.right.left, null );
		assertEquals ( "Right-hanging tree 1->2->3->4 incorrect.", (int)right.root.right.right.key, 3 );
		assertEquals ( "Right-hanging tree 1->2->3->4 incorrect.", right.root.right.right.left, null );
		assertEquals ( "Right-hanging tree 1->2->3->4 incorrect.", (int)right.root.right.right.right.key, 4 );
		assertEquals ( "Right-hanging tree 1->2->3->4 incorrect.", right.root.right.right.right.right, null );
		assertEquals ( "Right-hanging tree 1->2->3->4 incorrect.", right.root.right.right.right.left, null );
	}

	@Test
	public void testCountSmallerSimpsons() {
		setup();
		assertEquals ("Simpsons tree should have 0 elements < 1 !", simpsons.countSmaller(1), 0);
		assertEquals ("Simpsons tree should have 5 elements < 45 !", simpsons.countSmaller(45), 5);
		assertEquals ("Simpsons tree should have 3 elements < 17 !", simpsons.countSmaller(17), 3);
	}

	@Test
	public void testCountSmallerSimple() {
		setup();
		assertEquals ("Empty tree should have 0 elements < -1 !", empty.countSmaller(-1), 0);
		assertEquals ("Root-only tree should have 1 elements < 10 !", rootOnly.countSmaller(10), 1);
		assertEquals ("Root-only tree should have 0 elements < 0 !", rootOnly.countSmaller(0), 0);
	}
	
	@Test
	public void testCountUnbalanced() {
		setup();
		assertEquals ("Left-hanging tree 1<-2<-3<-4 should have 2 elements < 3 !", left.countSmaller(3), 2);
		assertEquals ("Left-hanging tree 1<-2<-3<-4 should have 0 elements < 1 !", left.countSmaller(1), 0);
		assertEquals ("Right-hanging tree 1->2->3->4 should have 0 elements < 1 !", right.countSmaller(1), 0);
		assertEquals ("Right-hanging tree 1->2->3->4 should have 2 elements < 3 !", right.countSmaller(3), 2);
	}

	@Test
	public void testCountDuplicates() {
		setup();
		assertEquals ("Tree with duplicates should have 2 elements < 3 !", duplicates.countSmaller(3), 2);
		assertEquals ("Tree with duplicates should have 1 elements < 2 !", duplicates.countSmaller(2), 1);
	}
	
}
